@@ -20,7 +20,7 @@ import (
 
 	"github.com/go-kit/kit/log"
 	"github.com/prometheus/common/model"
-	"github.com/prometheus/prometheus/util/testutil"
+	"github.com/stretchr/testify/require"
 	"gopkg.in/yaml.v2"
 )
 
@@ -36,21 +36,21 @@ role: tasks
 host: %s
 `, url)
 	var cfg SDConfig
-	testutil.Ok(t, yaml.Unmarshal([]byte(cfgString), &cfg))
+	require.NoError(t, yaml.Unmarshal([]byte(cfgString), &cfg))
 
 	d, err := NewDiscovery(&cfg, log.NewNopLogger())
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	ctx := context.Background()
 	tgs, err := d.refresh(ctx)
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
-	testutil.Equals(t, 1, len(tgs))
+	require.Equal(t, 1, len(tgs))
 
 	tg := tgs[0]
-	testutil.Assert(t, tg != nil, "tg should not be nil")
-	testutil.Assert(t, tg.Targets != nil, "tg.targets should not be nil")
-	testutil.Equals(t, 27, len(tg.Targets))
+	require.NotNil(t, tg)
+	require.NotNil(t, tg.Targets)
+	require.Equal(t, 27, len(tg.Targets))
 
 	for i, lbls := range []model.LabelSet{
 		{
@@ -787,7 +787,7 @@ host: %s
 		},
 	} {
 		t.Run(fmt.Sprintf("item %d", i), func(t *testing.T) {
-			testutil.Equals(t, lbls, tg.Targets[i])
+			require.Equal(t, lbls, tg.Targets[i])
 		})
 	}
 }
