@@ -18,7 +18,6 @@ import (
 	"encoding/json"
 
 	"github.com/prometheus/common/model"
-	"github.com/prometheus/prometheus/util/yaml"
 )
 
 // Group is a set of targets with a common label set(production , test, staging etc.).
@@ -40,9 +39,8 @@ func (tg Group) String() string {
 // UnmarshalYAML implements the yaml.Unmarshaler interface.
 func (tg *Group) UnmarshalYAML(unmarshal func(interface{}) error) error {
 	g := struct {
-		Targets []string               `yaml:"targets"`
-		Labels  model.LabelSet         `yaml:"labels"`
-		XXX     map[string]interface{} `yaml:",inline"`
+		Targets []string       `yaml:"targets"`
+		Labels  model.LabelSet `yaml:"labels"`
 	}{}
 	if err := unmarshal(&g); err != nil {
 		return err
@@ -54,7 +52,7 @@ func (tg *Group) UnmarshalYAML(unmarshal func(interface{}) error) error {
 		})
 	}
 	tg.Labels = g.Labels
-	return yaml.CheckOverflow(g.XXX, "static_config")
+	return nil
 }
 
 // MarshalYAML implements the yaml.Marshaler interface.
