@@ -14,7 +14,9 @@
 package rulefmt
 
 import (
+	"bytes"
 	"context"
+	"io"
 	"io/ioutil"
 	"strings"
 	"time"
@@ -184,7 +186,7 @@ func (r *RuleNode) Validate() (nodes []WrappedError) {
 	}
 
 	for k, v := range r.Labels {
-		if !model.LabelName(k).IsValid() {
+		if !model.LabelName(k).IsValid() || k == model.MetricNameLabel {
 			nodes = append(nodes, WrappedError{
 				err: errors.Errorf("invalid label name: %s", k),
 			})
@@ -267,8 +269,11 @@ func Parse(content []byte) (*RuleGroups, []error) {
 		errs   []error
 	)
 
-	err := yaml.Unmarshal(content, &groups)
-	if err != nil {
+	decoder := yaml.NewDecoder(bytes.NewReader(content))
+	decoder.KnownFields(true)
+	err := decoder.Decode(&groups)
+	// Ignore io.EOF which happens with empty input.
+	if err != nil && err != io.EOF {
 		errs = append(errs, err)
 	}
 	err = yaml.Unmarshal(content, &node)
