@@ -27,11 +27,13 @@ import (
 	"github.com/go-kit/kit/log"
 	"github.com/pkg/errors"
 	prom_testutil "github.com/prometheus/client_golang/prometheus/testutil"
+	"github.com/stretchr/testify/require"
+
 	"github.com/prometheus/prometheus/pkg/labels"
+	"github.com/prometheus/prometheus/tsdb/chunkenc"
 	"github.com/prometheus/prometheus/tsdb/chunks"
 	"github.com/prometheus/prometheus/tsdb/fileutil"
 	"github.com/prometheus/prometheus/tsdb/tombstones"
-	"github.com/prometheus/prometheus/util/testutil"
 )
 
 func TestSplitByRange(t *testing.T) {
@@ -133,7 +135,7 @@ func TestSplitByRange(t *testing.T) {
 			}
 		}
 
-		testutil.Equals(t, exp, splitByRange(blocks, c.trange))
+		require.Equal(t, exp, splitByRange(blocks, c.trange))
 	}
 }
 
@@ -157,7 +159,7 @@ func TestNoPanicFor0Tombstones(t *testing.T) {
 	}
 
 	c, err := NewLeveledCompactor(context.Background(), nil, nil, []int64{50}, nil)
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	c.plan(metas)
 }
@@ -171,7 +173,7 @@ func TestLeveledCompactor_plan(t *testing.T) {
 		540,
 		1620,
 	}, nil)
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	cases := map[string]struct {
 		metas    []dirMeta
@@ -364,8 +366,8 @@ func TestLeveledCompactor_plan(t *testing.T) {
 	for title, c := range cases {
 		if !t.Run(title, func(t *testing.T) {
 			res, err := compactor.plan(c.metas)
-			testutil.Ok(t, err)
-			testutil.Equals(t, c.expected, res)
+			require.NoError(t, err)
+			require.Equal(t, c.expected, res)
 		}) {
 			return
 		}
@@ -380,7 +382,7 @@ func TestRangeWithFailedCompactionWontGetSelected(t *testing.T) {
 		720,
 		2160,
 	}, nil)
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	cases := []struct {
 		metas []dirMeta
@@ -416,9 +418,9 @@ func TestRangeWithFailedCompactionWontGetSelected(t *testing.T) {
 	for _, c := range cases {
 		c.metas[1].meta.Compaction.Failed = true
 		res, err := compactor.plan(c.metas)
-		testutil.Ok(t, err)
+		require.NoError(t, err)
 
-		testutil.Equals(t, []string(nil), res)
+		require.Equal(t, []string(nil), res)
 	}
 }
 
@@ -430,17 +432,17 @@ func TestCompactionFailWillCleanUpTempDir(t *testing.T) {
 		720,
 		2160,
 	}, nil)
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	tmpdir, err := ioutil.TempDir("", "test")
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 	defer func() {
-		testutil.Ok(t, os.RemoveAll(tmpdir))
+		require.NoError(t, os.RemoveAll(tmpdir))
 	}()
 
-	testutil.NotOk(t, compactor.write(tmpdir, &BlockMeta{}, erringBReader{}))
-	_, err = os.Stat(filepath.Join(tmpdir, BlockMeta{}.ULID.String()) + ".tmp")
-	testutil.Assert(t, os.IsNotExist(err), "directory is not cleaned up")
+	require.Error(t, compactor.write(tmpdir, &BlockMeta{}, erringBReader{}))
+	_, err = os.Stat(filepath.Join(tmpdir, BlockMeta{}.ULID.String()) + tmpForCreationBlockDirSuffix)
+	require.True(t, os.IsNotExist(err), "directory is not cleaned up")
 }
 
 func metaRange(name string, mint, maxt int64, stats *BlockStats) dirMeta {
@@ -460,14 +462,30 @@ func (erringBReader) Index() (IndexReader, error)            { return nil, error
 func (erringBReader) Chunks() (ChunkReader, error)           { return nil, errors.New("chunks") }
 func (erringBReader) Tombstones() (tombstones.Reader, error) { return nil, errors.New("tombstones") }
 func (erringBReader) Meta() BlockMeta                        { return BlockMeta{} }
+func (erringBReader) Size() int64                            { return 0 }
 
 type nopChunkWriter struct{}
 
 func (nopChunkWriter) WriteChunks(chunks ...chunks.Meta) error { return nil }
 func (nopChunkWriter) Close() error                            { return nil }
 
+func samplesForRange(minTime, maxTime int64, maxSamplesPerChunk int) (ret [][]sample) {
+	var curr []sample
+	for i := minTime; i <= maxTime; i++ {
+		curr = append(curr, sample{t: i})
+		if len(curr) >= maxSamplesPerChunk {
+			ret = append(ret, curr)
+			curr = []sample{}
+		}
+	}
+	if len(curr) > 0 {
+		ret = append(ret, curr)
+	}
+	return ret
+}
+
 func TestCompaction_populateBlock(t *testing.T) {
-	var populateBlocksCases = []struct {
+	for _, tc := range []struct {
 		title              string
 		inputSeriesSamples [][]seriesSamples
 		compactMinTime     int64
@@ -483,11 +501,7 @@ func TestCompaction_populateBlock(t *testing.T) {
 		{
 			// Populate from single block without chunks. We expect these kind of series being ignored.
 			inputSeriesSamples: [][]seriesSamples{
-				{
-					{
-						lset: map[string]string{"a": "b"},
-					},
-				},
+				{{lset: map[string]string{"a": "b"}}},
 			},
 		},
 		{
@@ -546,6 +560,46 @@ func TestCompaction_populateBlock(t *testing.T) {
 				},
 			},
 		},
+		{
+			title: "Populate from two blocks; chunks with negative time.",
+			inputSeriesSamples: [][]seriesSamples{
+				{
+					{
+						lset:   map[string]string{"a": "b"},
+						chunks: [][]sample{{{t: 0}, {t: 10}}, {{t: 11}, {t: 20}}},
+					},
+					{
+						lset:   map[string]string{"a": "c"},
+						chunks: [][]sample{{{t: -11}, {t: -9}}, {{t: 10}, {t: 19}}},
+					},
+					{
+						// no-chunk series should be dropped.
+						lset: map[string]string{"a": "empty"},
+					},
+				},
+				{
+					{
+						lset:   map[string]string{"a": "b"},
+						chunks: [][]sample{{{t: 21}, {t: 30}}},
+					},
+					{
+						lset:   map[string]string{"a": "c"},
+						chunks: [][]sample{{{t: 40}, {t: 45}}},
+					},
+				},
+			},
+			compactMinTime: -11,
+			expSeriesSamples: []seriesSamples{
+				{
+					lset:   map[string]string{"a": "b"},
+					chunks: [][]sample{{{t: 0}, {t: 10}}, {{t: 11}, {t: 20}}, {{t: 21}, {t: 30}}},
+				},
+				{
+					lset:   map[string]string{"a": "c"},
+					chunks: [][]sample{{{t: -11}, {t: -9}}, {{t: 10}, {t: 19}}, {{t: 40}, {t: 45}}},
+				},
+			},
+		},
 		{
 			title: "Populate from two blocks showing that order is maintained.",
 			inputSeriesSamples: [][]seriesSamples{
@@ -636,8 +690,45 @@ func TestCompaction_populateBlock(t *testing.T) {
 				},
 			},
 		},
+		{
+			title: "Populate from two blocks 1:1 duplicated chunks; with negative timestamps.",
+			inputSeriesSamples: [][]seriesSamples{
+				{
+					{
+						lset:   map[string]string{"a": "1"},
+						chunks: [][]sample{{{t: 1}, {t: 2}}, {{t: 3}, {t: 4}}},
+					},
+					{
+						lset:   map[string]string{"a": "2"},
+						chunks: [][]sample{{{t: -3}, {t: -2}}, {{t: 1}, {t: 3}, {t: 4}}, {{t: 5}, {t: 6}}},
+					},
+				},
+				{
+					{
+						lset:   map[string]string{"a": "1"},
+						chunks: [][]sample{{{t: 3}, {t: 4}}},
+					},
+					{
+						lset:   map[string]string{"a": "2"},
+						chunks: [][]sample{{{t: 1}, {t: 3}, {t: 4}}, {{t: 7}, {t: 8}}},
+					},
+				},
+			},
+			compactMinTime: -3,
+			expSeriesSamples: []seriesSamples{
+				{
+					lset:   map[string]string{"a": "1"},
+					chunks: [][]sample{{{t: 1}, {t: 2}}, {{t: 3}, {t: 4}}},
+				},
+				{
+					lset:   map[string]string{"a": "2"},
+					chunks: [][]sample{{{t: -3}, {t: -2}}, {{t: 1}, {t: 3}, {t: 4}}, {{t: 5}, {t: 6}}, {{t: 7}, {t: 8}}},
+				},
+			},
+		},
 		{
 			// This should not happened because head block is making sure the chunks are not crossing block boundaries.
+			// We used to return error, but now chunk is trimmed.
 			title: "Populate from single block containing chunk outside of compact meta time range.",
 			inputSeriesSamples: [][]seriesSamples{
 				{
@@ -649,10 +740,15 @@ func TestCompaction_populateBlock(t *testing.T) {
 			},
 			compactMinTime: 0,
 			compactMaxTime: 20,
-			expErr:         errors.New("found chunk with minTime: 10 maxTime: 30 outside of compacted minTime: 0 maxTime: 20"),
+			expSeriesSamples: []seriesSamples{
+				{
+					lset:   map[string]string{"a": "b"},
+					chunks: [][]sample{{{t: 1}, {t: 2}}, {{t: 10}}},
+				},
+			},
 		},
 		{
-			// Introduced by https://github.com/prometheus/tsdb/issues/347.
+			// Introduced by https://github.com/prometheus/tsdb/issues/347. We used to return error, but now chunk is trimmed.
 			title: "Populate from single block containing extra chunk",
 			inputSeriesSamples: [][]seriesSamples{
 				{
@@ -664,7 +760,12 @@ func TestCompaction_populateBlock(t *testing.T) {
 			},
 			compactMinTime: 0,
 			compactMaxTime: 10,
-			expErr:         errors.New("found chunk with minTime: 10 maxTime: 20 outside of compacted minTime: 0 maxTime: 10"),
+			expSeriesSamples: []seriesSamples{
+				{
+					lset:   map[string]string{"a": "issue347"},
+					chunks: [][]sample{{{t: 1}, {t: 2}}},
+				},
+			},
 		},
 		{
 			// Deduplication expected.
@@ -693,54 +794,146 @@ func TestCompaction_populateBlock(t *testing.T) {
 		},
 		{
 			// Introduced by https://github.com/prometheus/tsdb/pull/539.
-			title: "Populate from three blocks that the last two are overlapping.",
+			title: "Populate from three overlapping blocks.",
 			inputSeriesSamples: [][]seriesSamples{
 				{
 					{
-						lset:   map[string]string{"before": "fix"},
-						chunks: [][]sample{{{t: 0}, {t: 10}, {t: 11}, {t: 20}}},
-					},
-					{
-						lset:   map[string]string{"after": "fix"},
-						chunks: [][]sample{{{t: 0}, {t: 10}, {t: 11}, {t: 20}}},
-					},
-				},
-				{
-					{
-						lset:   map[string]string{"before": "fix"},
+						lset:   map[string]string{"a": "overlap-all"},
 						chunks: [][]sample{{{t: 19}, {t: 30}}},
 					},
 					{
-						lset:   map[string]string{"after": "fix"},
+						lset:   map[string]string{"a": "overlap-beginning"},
+						chunks: [][]sample{{{t: 0}, {t: 5}}},
+					},
+					{
+						lset:   map[string]string{"a": "overlap-ending"},
 						chunks: [][]sample{{{t: 21}, {t: 30}}},
 					},
 				},
 				{
 					{
-						lset:   map[string]string{"before": "fix"},
+						lset:   map[string]string{"a": "overlap-all"},
+						chunks: [][]sample{{{t: 0}, {t: 10}, {t: 11}, {t: 20}}},
+					},
+					{
+						lset:   map[string]string{"a": "overlap-beginning"},
+						chunks: [][]sample{{{t: 0}, {t: 10}, {t: 12}, {t: 20}}},
+					},
+					{
+						lset:   map[string]string{"a": "overlap-ending"},
+						chunks: [][]sample{{{t: 0}, {t: 10}, {t: 13}, {t: 20}}},
+					},
+				},
+				{
+					{
+						lset:   map[string]string{"a": "overlap-all"},
 						chunks: [][]sample{{{t: 27}, {t: 35}}},
 					},
 					{
-						lset:   map[string]string{"after": "fix"},
+						lset:   map[string]string{"a": "overlap-ending"},
 						chunks: [][]sample{{{t: 27}, {t: 35}}},
 					},
 				},
 			},
 			expSeriesSamples: []seriesSamples{
 				{
-					lset:   map[string]string{"after": "fix"},
-					chunks: [][]sample{{{t: 0}, {t: 10}, {t: 11}, {t: 20}}, {{t: 21}, {t: 27}, {t: 30}, {t: 35}}},
+					lset:   map[string]string{"a": "overlap-all"},
+					chunks: [][]sample{{{t: 0}, {t: 10}, {t: 11}, {t: 19}, {t: 20}, {t: 27}, {t: 30}, {t: 35}}},
 				},
 				{
-					lset:   map[string]string{"before": "fix"},
-					chunks: [][]sample{{{t: 0}, {t: 10}, {t: 11}, {t: 19}, {t: 20}, {t: 27}, {t: 30}, {t: 35}}},
+					lset:   map[string]string{"a": "overlap-beginning"},
+					chunks: [][]sample{{{t: 0}, {t: 5}, {t: 10}, {t: 12}, {t: 20}}},
+				},
+				{
+					lset:   map[string]string{"a": "overlap-ending"},
+					chunks: [][]sample{{{t: 0}, {t: 10}, {t: 13}, {t: 20}}, {{t: 21}, {t: 27}, {t: 30}, {t: 35}}},
 				},
 			},
 		},
-	}
-
-	for _, tc := range populateBlocksCases {
-		if ok := t.Run(tc.title, func(t *testing.T) {
+		{
+			title: "Populate from three partially overlapping blocks with few full chunks.",
+			inputSeriesSamples: [][]seriesSamples{
+				{
+					{
+						lset:   map[string]string{"a": "1", "b": "1"},
+						chunks: samplesForRange(0, 659, 120), // 5 chunks and half.
+					},
+					{
+						lset:   map[string]string{"a": "1", "b": "2"},
+						chunks: samplesForRange(0, 659, 120),
+					},
+				},
+				{
+					{
+						lset:   map[string]string{"a": "1", "b": "2"},
+						chunks: samplesForRange(480, 1199, 120), // two chunks overlapping with previous, two non overlapping and two overlapping with next block.
+					},
+					{
+						lset:   map[string]string{"a": "1", "b": "3"},
+						chunks: samplesForRange(480, 1199, 120),
+					},
+				},
+				{
+					{
+						lset:   map[string]string{"a": "1", "b": "2"},
+						chunks: samplesForRange(960, 1499, 120), // 5 chunks and half.
+					},
+					{
+						lset:   map[string]string{"a": "1", "b": "4"},
+						chunks: samplesForRange(960, 1499, 120),
+					},
+				},
+			},
+			expSeriesSamples: []seriesSamples{
+				{
+					lset:   map[string]string{"a": "1", "b": "1"},
+					chunks: samplesForRange(0, 659, 120),
+				},
+				{
+					lset:   map[string]string{"a": "1", "b": "2"},
+					chunks: samplesForRange(0, 1499, 120),
+				},
+				{
+					lset:   map[string]string{"a": "1", "b": "3"},
+					chunks: samplesForRange(480, 1199, 120),
+				},
+				{
+					lset:   map[string]string{"a": "1", "b": "4"},
+					chunks: samplesForRange(960, 1499, 120),
+				},
+			},
+		},
+		{
+			title: "Populate from three partially overlapping blocks with chunks that are expected to merge into single big chunks.",
+			inputSeriesSamples: [][]seriesSamples{
+				{
+					{
+						lset:   map[string]string{"a": "1", "b": "2"},
+						chunks: [][]sample{{{t: 0}, {t: 6902464}}, {{t: 6961968}, {t: 7080976}}},
+					},
+				},
+				{
+					{
+						lset:   map[string]string{"a": "1", "b": "2"},
+						chunks: [][]sample{{{t: 3600000}, {t: 13953696}}, {{t: 14042952}, {t: 14221464}}},
+					},
+				},
+				{
+					{
+						lset:   map[string]string{"a": "1", "b": "2"},
+						chunks: [][]sample{{{t: 10800000}, {t: 14251232}}, {{t: 14280984}, {t: 14340488}}},
+					},
+				},
+			},
+			expSeriesSamples: []seriesSamples{
+				{
+					lset:   map[string]string{"a": "1", "b": "2"},
+					chunks: [][]sample{{{t: 0}, {t: 3600000}, {t: 6902464}, {t: 6961968}, {t: 7080976}, {t: 10800000}, {t: 13953696}, {t: 14042952}, {t: 14221464}, {t: 14251232}}, {{t: 14280984}, {t: 14340488}}},
+				},
+			},
+		},
+	} {
+		t.Run(tc.title, func(t *testing.T) {
 			blocks := make([]BlockReader, 0, len(tc.inputSeriesSamples))
 			for _, b := range tc.inputSeriesSamples {
 				ir, cr, mint, maxt := createIdxChkReaders(t, b)
@@ -748,7 +941,7 @@ func TestCompaction_populateBlock(t *testing.T) {
 			}
 
 			c, err := NewLeveledCompactor(context.Background(), nil, nil, []int64{0}, nil)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 
 			meta := &BlockMeta{
 				MinTime: tc.compactMinTime,
@@ -761,28 +954,53 @@ func TestCompaction_populateBlock(t *testing.T) {
 			iw := &mockIndexWriter{}
 			err = c.populateBlock(blocks, meta, iw, nopChunkWriter{})
 			if tc.expErr != nil {
-				testutil.NotOk(t, err)
-				testutil.Equals(t, tc.expErr.Error(), err.Error())
+				require.Error(t, err)
+				require.Equal(t, tc.expErr.Error(), err.Error())
 				return
 			}
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 
-			testutil.Equals(t, tc.expSeriesSamples, iw.series)
+			// Check if response is expected and chunk is valid.
+			var raw []seriesSamples
+			for _, s := range iw.seriesChunks {
+				ss := seriesSamples{lset: s.l.Map()}
+				var iter chunkenc.Iterator
+				for _, chk := range s.chunks {
+					var (
+						samples       = make([]sample, 0, chk.Chunk.NumSamples())
+						iter          = chk.Chunk.Iterator(iter)
+						firstTs int64 = math.MaxInt64
+						s       sample
+					)
+					for iter.Next() {
+						s.t, s.v = iter.At()
+						if firstTs == math.MaxInt64 {
+							firstTs = s.t
+						}
+						samples = append(samples, s)
+					}
+
+					// Check if chunk has correct min, max times.
+					require.Equal(t, firstTs, chk.MinTime, "chunk Meta %v does not match the first encoded sample timestamp: %v", chk, firstTs)
+					require.Equal(t, s.t, chk.MaxTime, "chunk Meta %v does not match the last encoded sample timestamp %v", chk, s.t)
+
+					require.NoError(t, iter.Err())
+					ss.chunks = append(ss.chunks, samples)
+				}
+				raw = append(raw, ss)
+			}
+			require.Equal(t, tc.expSeriesSamples, raw)
 
 			// Check if stats are calculated properly.
-			s := BlockStats{
-				NumSeries: uint64(len(tc.expSeriesSamples)),
-			}
+			s := BlockStats{NumSeries: uint64(len(tc.expSeriesSamples))}
 			for _, series := range tc.expSeriesSamples {
 				s.NumChunks += uint64(len(series.chunks))
 				for _, chk := range series.chunks {
 					s.NumSamples += uint64(len(chk))
 				}
 			}
-			testutil.Equals(t, s, meta.Stats)
-		}); !ok {
-			return
-		}
+			require.Equal(t, s, meta.Stats)
+		})
 	}
 }
 
@@ -831,30 +1049,30 @@ func BenchmarkCompaction(b *testing.B) {
 		nBlocks := len(c.ranges)
 		b.Run(fmt.Sprintf("type=%s,blocks=%d,series=%d,samplesPerSeriesPerBlock=%d", c.compactionType, nBlocks, nSeries, c.ranges[0][1]-c.ranges[0][0]+1), func(b *testing.B) {
 			dir, err := ioutil.TempDir("", "bench_compaction")
-			testutil.Ok(b, err)
+			require.NoError(b, err)
 			defer func() {
-				testutil.Ok(b, os.RemoveAll(dir))
+				require.NoError(b, os.RemoveAll(dir))
 			}()
 			blockDirs := make([]string, 0, len(c.ranges))
 			var blocks []*Block
 			for _, r := range c.ranges {
 				block, err := OpenBlock(nil, createBlock(b, dir, genSeries(nSeries, 10, r[0], r[1])), nil)
-				testutil.Ok(b, err)
+				require.NoError(b, err)
 				blocks = append(blocks, block)
 				defer func() {
-					testutil.Ok(b, block.Close())
+					require.NoError(b, block.Close())
 				}()
 				blockDirs = append(blockDirs, block.Dir())
 			}
 
 			c, err := NewLeveledCompactor(context.Background(), nil, log.NewNopLogger(), []int64{0}, nil)
-			testutil.Ok(b, err)
+			require.NoError(b, err)
 
 			b.ResetTimer()
 			b.ReportAllocs()
 			for i := 0; i < b.N; i++ {
 				_, err = c.Compact(dir, blockDirs, blocks)
-				testutil.Ok(b, err)
+				require.NoError(b, err)
 			}
 		})
 	}
@@ -862,27 +1080,27 @@ func BenchmarkCompaction(b *testing.B) {
 
 func BenchmarkCompactionFromHead(b *testing.B) {
 	dir, err := ioutil.TempDir("", "bench_compaction_from_head")
-	testutil.Ok(b, err)
+	require.NoError(b, err)
 	defer func() {
-		testutil.Ok(b, os.RemoveAll(dir))
+		require.NoError(b, os.RemoveAll(dir))
 	}()
 	totalSeries := 100000
 	for labelNames := 1; labelNames < totalSeries; labelNames *= 10 {
 		labelValues := totalSeries / labelNames
 		b.Run(fmt.Sprintf("labelnames=%d,labelvalues=%d", labelNames, labelValues), func(b *testing.B) {
 			chunkDir, err := ioutil.TempDir("", "chunk_dir")
-			testutil.Ok(b, err)
+			require.NoError(b, err)
 			defer func() {
-				testutil.Ok(b, os.RemoveAll(chunkDir))
+				require.NoError(b, os.RemoveAll(chunkDir))
 			}()
-			h, err := NewHead(nil, nil, nil, 1000, chunkDir, nil, DefaultStripeSize, nil)
-			testutil.Ok(b, err)
+			h, err := NewHead(nil, nil, nil, 1000, chunkDir, nil, chunks.DefaultWriteBufferSize, DefaultStripeSize, nil)
+			require.NoError(b, err)
 			for ln := 0; ln < labelNames; ln++ {
-				app := h.Appender()
+				app := h.Appender(context.Background())
 				for lv := 0; lv < labelValues; lv++ {
 					app.Add(labels.FromStrings(fmt.Sprintf("%d", ln), fmt.Sprintf("%d%s%d", lv, postingsBenchSuffix, ln)), 0, 0)
 				}
-				testutil.Ok(b, app.Commit())
+				require.NoError(b, app.Commit())
 			}
 
 			b.ResetTimer()
@@ -900,10 +1118,9 @@ func BenchmarkCompactionFromHead(b *testing.B) {
 // This is needed for unit tests that rely on
 // checking state before and after a compaction.
 func TestDisableAutoCompactions(t *testing.T) {
-	db, closeFn := openTestDB(t, nil, nil)
+	db := openTestDB(t, nil, nil)
 	defer func() {
-		testutil.Ok(t, db.Close())
-		closeFn()
+		require.NoError(t, db.Close())
 	}()
 
 	blockRange := db.compactor.(*LeveledCompactor).ranges[0]
@@ -912,14 +1129,14 @@ func TestDisableAutoCompactions(t *testing.T) {
 	// Trigger a compaction to check that it was skipped and
 	// no new blocks were created when compaction is disabled.
 	db.DisableCompactions()
-	app := db.Appender()
+	app := db.Appender(context.Background())
 	for i := int64(0); i < 3; i++ {
 		_, err := app.Add(label, i*blockRange, 0)
-		testutil.Ok(t, err)
+		require.NoError(t, err)
 		_, err = app.Add(label, i*blockRange+1000, 0)
-		testutil.Ok(t, err)
+		require.NoError(t, err)
 	}
-	testutil.Ok(t, app.Commit())
+	require.NoError(t, app.Commit())
 
 	select {
 	case db.compactc <- struct{}{}:
@@ -933,8 +1150,8 @@ func TestDisableAutoCompactions(t *testing.T) {
 		time.Sleep(10 * time.Millisecond)
 	}
 
-	testutil.Assert(t, prom_testutil.ToFloat64(db.metrics.compactionsSkipped) > 0.0, "No compaction was skipped after the set timeout.")
-	testutil.Equals(t, 0, len(db.blocks))
+	require.Greater(t, prom_testutil.ToFloat64(db.metrics.compactionsSkipped), 0.0, "No compaction was skipped after the set timeout.")
+	require.Equal(t, 0, len(db.blocks))
 
 	// Enable the compaction, trigger it and check that the block is persisted.
 	db.EnableCompactions()
@@ -948,16 +1165,16 @@ func TestDisableAutoCompactions(t *testing.T) {
 		}
 		time.Sleep(100 * time.Millisecond)
 	}
-	testutil.Assert(t, len(db.Blocks()) > 0, "No block was persisted after the set timeout.")
+	require.Greater(t, len(db.Blocks()), 0, "No block was persisted after the set timeout.")
 }
 
 // TestCancelCompactions ensures that when the db is closed
 // any running compaction is cancelled to unblock closing the db.
 func TestCancelCompactions(t *testing.T) {
 	tmpdir, err := ioutil.TempDir("", "testCancelCompaction")
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 	defer func() {
-		testutil.Ok(t, os.RemoveAll(tmpdir))
+		require.NoError(t, os.RemoveAll(tmpdir))
 	}()
 
 	// Create some blocks to fall within the compaction range.
@@ -968,18 +1185,18 @@ func TestCancelCompactions(t *testing.T) {
 	// Copy the db so we have an exact copy to compare compaction times.
 	tmpdirCopy := tmpdir + "Copy"
 	err = fileutil.CopyDirs(tmpdir, tmpdirCopy)
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 	defer func() {
-		testutil.Ok(t, os.RemoveAll(tmpdirCopy))
+		require.NoError(t, os.RemoveAll(tmpdirCopy))
 	}()
 
 	// Measure the compaction time without interrupting it.
 	var timeCompactionUninterrupted time.Duration
 	{
 		db, err := open(tmpdir, log.NewNopLogger(), nil, DefaultOptions(), []int64{1, 2000})
-		testutil.Ok(t, err)
-		testutil.Equals(t, 3, len(db.Blocks()), "initial block count mismatch")
-		testutil.Equals(t, 0.0, prom_testutil.ToFloat64(db.compactor.(*LeveledCompactor).metrics.ran), "initial compaction counter mismatch")
+		require.NoError(t, err)
+		require.Equal(t, 3, len(db.Blocks()), "initial block count mismatch")
+		require.Equal(t, 0.0, prom_testutil.ToFloat64(db.compactor.(*LeveledCompactor).metrics.ran), "initial compaction counter mismatch")
 		db.compactc <- struct{}{} // Trigger a compaction.
 		var start time.Time
 		for prom_testutil.ToFloat64(db.compactor.(*LeveledCompactor).metrics.populatingBlocks) <= 0 {
@@ -992,14 +1209,14 @@ func TestCancelCompactions(t *testing.T) {
 		}
 		timeCompactionUninterrupted = time.Since(start)
 
-		testutil.Ok(t, db.Close())
+		require.NoError(t, db.Close())
 	}
 	// Measure the compaction time when closing the db in the middle of compaction.
 	{
 		db, err := open(tmpdirCopy, log.NewNopLogger(), nil, DefaultOptions(), []int64{1, 2000})
-		testutil.Ok(t, err)
-		testutil.Equals(t, 3, len(db.Blocks()), "initial block count mismatch")
-		testutil.Equals(t, 0.0, prom_testutil.ToFloat64(db.compactor.(*LeveledCompactor).metrics.ran), "initial compaction counter mismatch")
+		require.NoError(t, err)
+		require.Equal(t, 3, len(db.Blocks()), "initial block count mismatch")
+		require.Equal(t, 0.0, prom_testutil.ToFloat64(db.compactor.(*LeveledCompactor).metrics.ran), "initial compaction counter mismatch")
 		db.compactc <- struct{}{} // Trigger a compaction.
 		dbClosed := make(chan struct{})
 
@@ -1007,7 +1224,7 @@ func TestCancelCompactions(t *testing.T) {
 			time.Sleep(3 * time.Millisecond)
 		}
 		go func() {
-			testutil.Ok(t, db.Close())
+			require.NoError(t, db.Close())
 			close(dbClosed)
 		}()
 
@@ -1015,11 +1232,11 @@ func TestCancelCompactions(t *testing.T) {
 		<-dbClosed
 		actT := time.Since(start)
 		expT := time.Duration(timeCompactionUninterrupted / 2) // Closing the db in the middle of compaction should less than half the time.
-		testutil.Assert(t, actT < expT, "closing the db took more than expected. exp: <%v, act: %v", expT, actT)
+		require.True(t, actT < expT, "closing the db took more than expected. exp: <%v, act: %v", expT, actT)
 	}
 }
 
-// TestDeleteCompactionBlockAfterFailedReload ensures that a failed reload immediately after a compaction
+// TestDeleteCompactionBlockAfterFailedReload ensures that a failed reloadBlocks immediately after a compaction
 // deletes the resulting block to avoid creatings blocks with the same time range.
 func TestDeleteCompactionBlockAfterFailedReload(t *testing.T) {
 	tests := map[string]func(*DB) int{
@@ -1028,14 +1245,14 @@ func TestDeleteCompactionBlockAfterFailedReload(t *testing.T) {
 			defaultLabel := labels.FromStrings("foo", "bar")
 
 			// Add some data to the head that is enough to trigger a compaction.
-			app := db.Appender()
+			app := db.Appender(context.Background())
 			_, err := app.Add(defaultLabel, 1, 0)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 			_, err = app.Add(defaultLabel, 2, 0)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 			_, err = app.Add(defaultLabel, 3+rangeToTriggerCompaction, 0)
-			testutil.Ok(t, err)
-			testutil.Ok(t, app.Commit())
+			require.NoError(t, err)
+			require.NoError(t, app.Commit())
 
 			return 0
 		},
@@ -1048,8 +1265,8 @@ func TestDeleteCompactionBlockAfterFailedReload(t *testing.T) {
 			for _, m := range blocks {
 				createBlock(t, db.Dir(), genSeries(1, 1, m.MinTime, m.MaxTime))
 			}
-			testutil.Ok(t, db.reload())
-			testutil.Equals(t, len(blocks), len(db.Blocks()), "unexpected block count after a reload")
+			require.NoError(t, db.reload())
+			require.Equal(t, len(blocks), len(db.Blocks()), "unexpected block count after a reloadBlocks")
 
 			return len(blocks)
 		},
@@ -1057,38 +1274,37 @@ func TestDeleteCompactionBlockAfterFailedReload(t *testing.T) {
 
 	for title, bootStrap := range tests {
 		t.Run(title, func(t *testing.T) {
-			db, closeFn := openTestDB(t, nil, []int64{1, 100})
+			db := openTestDB(t, nil, []int64{1, 100})
 			defer func() {
-				testutil.Ok(t, db.Close())
-				closeFn()
+				require.NoError(t, db.Close())
 			}()
 			db.DisableCompactions()
 
 			expBlocks := bootStrap(db)
 
-			// Create a block that will trigger the reload to fail.
+			// Create a block that will trigger the reloadBlocks to fail.
 			blockPath := createBlock(t, db.Dir(), genSeries(1, 1, 200, 300))
 			lastBlockIndex := path.Join(blockPath, indexFilename)
 			actBlocks, err := blockDirs(db.Dir())
-			testutil.Ok(t, err)
-			testutil.Equals(t, expBlocks, len(actBlocks)-1) // -1 to exclude the corrupted block.
-			testutil.Ok(t, os.RemoveAll(lastBlockIndex))    // Corrupt the block by removing the index file.
+			require.NoError(t, err)
+			require.Equal(t, expBlocks, len(actBlocks)-1)    // -1 to exclude the corrupted block.
+			require.NoError(t, os.RemoveAll(lastBlockIndex)) // Corrupt the block by removing the index file.
 
-			testutil.Equals(t, 0.0, prom_testutil.ToFloat64(db.metrics.reloadsFailed), "initial 'failed db reload' count metrics mismatch")
-			testutil.Equals(t, 0.0, prom_testutil.ToFloat64(db.compactor.(*LeveledCompactor).metrics.ran), "initial `compactions` count metric mismatch")
-			testutil.Equals(t, 0.0, prom_testutil.ToFloat64(db.metrics.compactionsFailed), "initial `compactions failed` count metric mismatch")
+			require.Equal(t, 0.0, prom_testutil.ToFloat64(db.metrics.reloadsFailed), "initial 'failed db reloadBlocks' count metrics mismatch")
+			require.Equal(t, 0.0, prom_testutil.ToFloat64(db.compactor.(*LeveledCompactor).metrics.ran), "initial `compactions` count metric mismatch")
+			require.Equal(t, 0.0, prom_testutil.ToFloat64(db.metrics.compactionsFailed), "initial `compactions failed` count metric mismatch")
 
 			// Do the compaction and check the metrics.
-			// Compaction should succeed, but the reload should fail and
+			// Compaction should succeed, but the reloadBlocks should fail and
 			// the new block created from the compaction should be deleted.
-			testutil.NotOk(t, db.Compact())
-			testutil.Equals(t, 1.0, prom_testutil.ToFloat64(db.metrics.reloadsFailed), "'failed db reload' count metrics mismatch")
-			testutil.Equals(t, 1.0, prom_testutil.ToFloat64(db.compactor.(*LeveledCompactor).metrics.ran), "`compaction` count metric mismatch")
-			testutil.Equals(t, 1.0, prom_testutil.ToFloat64(db.metrics.compactionsFailed), "`compactions failed` count metric mismatch")
+			require.Error(t, db.Compact())
+			require.Equal(t, 1.0, prom_testutil.ToFloat64(db.metrics.reloadsFailed), "'failed db reloadBlocks' count metrics mismatch")
+			require.Equal(t, 1.0, prom_testutil.ToFloat64(db.compactor.(*LeveledCompactor).metrics.ran), "`compaction` count metric mismatch")
+			require.Equal(t, 1.0, prom_testutil.ToFloat64(db.metrics.compactionsFailed), "`compactions failed` count metric mismatch")
 
 			actBlocks, err = blockDirs(db.Dir())
-			testutil.Ok(t, err)
-			testutil.Equals(t, expBlocks, len(actBlocks)-1, "block count should be the same as before the compaction") // -1 to exclude the corrupted block.
+			require.NoError(t, err)
+			require.Equal(t, expBlocks, len(actBlocks)-1, "block count should be the same as before the compaction") // -1 to exclude the corrupted block.
 		})
 	}
 }
