@@ -14,14 +14,17 @@
 package tsdb
 
 import (
+	"context"
 	"fmt"
 	"io/ioutil"
 	"os"
 	"strconv"
 	"testing"
 
+	"github.com/stretchr/testify/require"
+
 	"github.com/prometheus/prometheus/pkg/labels"
-	"github.com/prometheus/prometheus/util/testutil"
+	"github.com/prometheus/prometheus/tsdb/chunks"
 )
 
 // Make entries ~50B in size, to emulate real-world high cardinality.
@@ -31,17 +34,17 @@ const (
 
 func BenchmarkPostingsForMatchers(b *testing.B) {
 	chunkDir, err := ioutil.TempDir("", "chunk_dir")
-	testutil.Ok(b, err)
+	require.NoError(b, err)
 	defer func() {
-		testutil.Ok(b, os.RemoveAll(chunkDir))
+		require.NoError(b, os.RemoveAll(chunkDir))
 	}()
-	h, err := NewHead(nil, nil, nil, 1000, chunkDir, nil, DefaultStripeSize, nil)
-	testutil.Ok(b, err)
+	h, err := NewHead(nil, nil, nil, 1000, chunkDir, nil, chunks.DefaultWriteBufferSize, DefaultStripeSize, nil)
+	require.NoError(b, err)
 	defer func() {
-		testutil.Ok(b, h.Close())
+		require.NoError(b, h.Close())
 	}()
 
-	app := h.Appender()
+	app := h.Appender(context.Background())
 	addSeries := func(l labels.Labels) {
 		app.Add(l, 0, 0)
 	}
@@ -56,28 +59,28 @@ func BenchmarkPostingsForMatchers(b *testing.B) {
 			addSeries(labels.FromStrings("i", strconv.Itoa(i)+postingsBenchSuffix, "n", "2_"+strconv.Itoa(n)+postingsBenchSuffix, "j", "foo"))
 		}
 	}
-	testutil.Ok(b, app.Commit())
+	require.NoError(b, app.Commit())
 
 	ir, err := h.Index()
-	testutil.Ok(b, err)
+	require.NoError(b, err)
 	b.Run("Head", func(b *testing.B) {
 		benchmarkPostingsForMatchers(b, ir)
 	})
 
 	tmpdir, err := ioutil.TempDir("", "test_benchpostingsformatchers")
-	testutil.Ok(b, err)
+	require.NoError(b, err)
 	defer func() {
-		testutil.Ok(b, os.RemoveAll(tmpdir))
+		require.NoError(b, os.RemoveAll(tmpdir))
 	}()
 
 	blockdir := createBlockFromHead(b, tmpdir, h)
 	block, err := OpenBlock(nil, blockdir, nil)
-	testutil.Ok(b, err)
+	require.NoError(b, err)
 	defer func() {
-		testutil.Ok(b, block.Close())
+		require.NoError(b, block.Close())
 	}()
 	ir, err = block.Index()
-	testutil.Ok(b, err)
+	require.NoError(b, err)
 	defer ir.Close()
 	b.Run("Block", func(b *testing.B) {
 		benchmarkPostingsForMatchers(b, ir)
@@ -132,7 +135,7 @@ func benchmarkPostingsForMatchers(b *testing.B, ir IndexReader) {
 		b.Run(c.name, func(b *testing.B) {
 			for i := 0; i < b.N; i++ {
 				_, err := PostingsForMatchers(ir, c.matchers...)
-				testutil.Ok(b, err)
+				require.NoError(b, err)
 			}
 		})
 	}
@@ -140,33 +143,33 @@ func benchmarkPostingsForMatchers(b *testing.B, ir IndexReader) {
 
 func BenchmarkQuerierSelect(b *testing.B) {
 	chunkDir, err := ioutil.TempDir("", "chunk_dir")
-	testutil.Ok(b, err)
+	require.NoError(b, err)
 	defer func() {
-		testutil.Ok(b, os.RemoveAll(chunkDir))
+		require.NoError(b, os.RemoveAll(chunkDir))
 	}()
-	h, err := NewHead(nil, nil, nil, 1000, chunkDir, nil, DefaultStripeSize, nil)
-	testutil.Ok(b, err)
+	h, err := NewHead(nil, nil, nil, 1000, chunkDir, nil, chunks.DefaultWriteBufferSize, DefaultStripeSize, nil)
+	require.NoError(b, err)
 	defer h.Close()
-	app := h.Appender()
+	app := h.Appender(context.Background())
 	numSeries := 1000000
 	for i := 0; i < numSeries; i++ {
 		app.Add(labels.FromStrings("foo", "bar", "i", fmt.Sprintf("%d%s", i, postingsBenchSuffix)), int64(i), 0)
 	}
-	testutil.Ok(b, app.Commit())
+	require.NoError(b, app.Commit())
 
 	bench := func(b *testing.B, br BlockReader, sorted bool) {
 		matcher := labels.MustNewMatcher(labels.MatchEqual, "foo", "bar")
 		for s := 1; s <= numSeries; s *= 10 {
 			b.Run(fmt.Sprintf("%dof%d", s, numSeries), func(b *testing.B) {
 				q, err := NewBlockQuerier(br, 0, int64(s-1))
-				testutil.Ok(b, err)
+				require.NoError(b, err)
 
 				b.ResetTimer()
 				for i := 0; i < b.N; i++ {
 					ss := q.Select(sorted, nil, matcher)
 					for ss.Next() {
 					}
-					testutil.Ok(b, ss.Err())
+					require.NoError(b, ss.Err())
 				}
 				q.Close()
 			})
@@ -181,16 +184,16 @@ func BenchmarkQuerierSelect(b *testing.B) {
 	})
 
 	tmpdir, err := ioutil.TempDir("", "test_benchquerierselect")
-	testutil.Ok(b, err)
+	require.NoError(b, err)
 	defer func() {
-		testutil.Ok(b, os.RemoveAll(tmpdir))
+		require.NoError(b, os.RemoveAll(tmpdir))
 	}()
 
 	blockdir := createBlockFromHead(b, tmpdir, h)
 	block, err := OpenBlock(nil, blockdir, nil)
-	testutil.Ok(b, err)
+	require.NoError(b, err)
 	defer func() {
-		testutil.Ok(b, block.Close())
+		require.NoError(b, block.Close())
 	}()
 
 	b.Run("Block", func(b *testing.B) {
