@@ -14,7 +14,6 @@ package wal
 
 import (
 	"fmt"
-	"github.com/go-kit/kit/log"
 	"io/ioutil"
 	"math/rand"
 	"os"
@@ -23,10 +22,12 @@ import (
 	"testing"
 	"time"
 
+	"github.com/go-kit/kit/log"
 	"github.com/prometheus/client_golang/prometheus"
+	"github.com/stretchr/testify/require"
+
 	"github.com/prometheus/prometheus/pkg/labels"
 	"github.com/prometheus/prometheus/tsdb/record"
-	"github.com/prometheus/prometheus/util/testutil"
 )
 
 var defaultRetryInterval = 100 * time.Millisecond
@@ -99,16 +100,21 @@ func TestTailSamples(t *testing.T) {
 			now := time.Now()
 
 			dir, err := ioutil.TempDir("", "readCheckpoint")
-			testutil.Ok(t, err)
-			defer os.RemoveAll(dir)
+			require.NoError(t, err)
+			defer func() {
+				require.NoError(t, os.RemoveAll(dir))
+			}()
 
 			wdir := path.Join(dir, "wal")
 			err = os.Mkdir(wdir, 0777)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 
 			enc := record.Encoder{}
 			w, err := NewSize(nil, nil, wdir, 128*pageSize, compress)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
+			defer func() {
+				require.NoError(t, w.Close())
+			}()
 
 			// Write to the initial segment then checkpoint.
 			for i := 0; i < seriesCount; i++ {
@@ -119,7 +125,7 @@ func TestTailSamples(t *testing.T) {
 						Labels: labels.Labels{labels.Label{Name: "__name__", Value: fmt.Sprintf("metric_%d", i)}},
 					},
 				}, nil)
-				testutil.Ok(t, w.Log(series))
+				require.NoError(t, w.Log(series))
 
 				for j := 0; j < samplesCount; j++ {
 					inner := rand.Intn(ref + 1)
@@ -130,13 +136,13 @@ func TestTailSamples(t *testing.T) {
 							V:   float64(i),
 						},
 					}, nil)
-					testutil.Ok(t, w.Log(sample))
+					require.NoError(t, w.Log(sample))
 				}
 			}
 
 			// Start read after checkpoint, no more data written.
-			first, last, err := w.Segments()
-			testutil.Ok(t, err)
+			first, last, err := Segments(w.Dir())
+			require.NoError(t, err)
 
 			wt := newWriteToMock()
 			watcher := NewWatcher(wMetrics, nil, nil, "", wt, dir)
@@ -146,7 +152,7 @@ func TestTailSamples(t *testing.T) {
 			watcher.setMetrics()
 			for i := first; i <= last; i++ {
 				segment, err := OpenReadSegment(SegmentName(watcher.walDir, i))
-				testutil.Ok(t, err)
+				require.NoError(t, err)
 				defer segment.Close()
 
 				reader := NewLiveReader(nil, NewLiveReaderMetrics(nil), segment)
@@ -159,8 +165,8 @@ func TestTailSamples(t *testing.T) {
 			retry(t, defaultRetryInterval, defaultRetries, func() bool {
 				return wt.checkNumLabels() >= expectedSeries
 			})
-			testutil.Equals(t, expectedSeries, wt.checkNumLabels())
-			testutil.Equals(t, expectedSamples, wt.samplesAppended)
+			require.Equal(t, expectedSeries, wt.checkNumLabels())
+			require.Equal(t, expectedSamples, wt.samplesAppended)
 		})
 	}
 }
@@ -173,14 +179,19 @@ func TestReadToEndNoCheckpoint(t *testing.T) {
 	for _, compress := range []bool{false, true} {
 		t.Run(fmt.Sprintf("compress=%t", compress), func(t *testing.T) {
 			dir, err := ioutil.TempDir("", "readToEnd_noCheckpoint")
-			testutil.Ok(t, err)
-			defer os.RemoveAll(dir)
+			require.NoError(t, err)
+			defer func() {
+				require.NoError(t, os.RemoveAll(dir))
+			}()
 			wdir := path.Join(dir, "wal")
 			err = os.Mkdir(wdir, 0777)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 
 			w, err := NewSize(nil, nil, wdir, 128*pageSize, compress)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
+			defer func() {
+				require.NoError(t, w.Close())
+			}()
 
 			var recs [][]byte
 
@@ -207,15 +218,15 @@ func TestReadToEndNoCheckpoint(t *testing.T) {
 
 					// Randomly batch up records.
 					if rand.Intn(4) < 3 {
-						testutil.Ok(t, w.Log(recs...))
+						require.NoError(t, w.Log(recs...))
 						recs = recs[:0]
 					}
 				}
 			}
-			testutil.Ok(t, w.Log(recs...))
+			require.NoError(t, w.Log(recs...))
 
-			_, _, err = w.Segments()
-			testutil.Ok(t, err)
+			_, _, err = Segments(w.Dir())
+			require.NoError(t, err)
 
 			wt := newWriteToMock()
 			watcher := NewWatcher(wMetrics, nil, nil, "", wt, dir)
@@ -226,7 +237,7 @@ func TestReadToEndNoCheckpoint(t *testing.T) {
 				return wt.checkNumLabels() >= expected
 			})
 			watcher.Stop()
-			testutil.Equals(t, expected, wt.checkNumLabels())
+			require.Equal(t, expected, wt.checkNumLabels())
 		})
 	}
 }
@@ -241,16 +252,21 @@ func TestReadToEndWithCheckpoint(t *testing.T) {
 	for _, compress := range []bool{false, true} {
 		t.Run(fmt.Sprintf("compress=%t", compress), func(t *testing.T) {
 			dir, err := ioutil.TempDir("", "readToEnd_withCheckpoint")
-			testutil.Ok(t, err)
-			defer os.RemoveAll(dir)
+			require.NoError(t, err)
+			defer func() {
+				require.NoError(t, os.RemoveAll(dir))
+			}()
 
 			wdir := path.Join(dir, "wal")
 			err = os.Mkdir(wdir, 0777)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 
 			enc := record.Encoder{}
 			w, err := NewSize(nil, nil, wdir, segmentSize, compress)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
+			defer func() {
+				require.NoError(t, w.Close())
+			}()
 
 			// Write to the initial segment then checkpoint.
 			for i := 0; i < seriesCount; i++ {
@@ -261,7 +277,9 @@ func TestReadToEndWithCheckpoint(t *testing.T) {
 						Labels: labels.Labels{labels.Label{Name: "__name__", Value: fmt.Sprintf("metric_%d", i)}},
 					},
 				}, nil)
-				testutil.Ok(t, w.Log(series))
+				require.NoError(t, w.Log(series))
+				// Add in an unknown record type, which should be ignored.
+				require.NoError(t, w.Log([]byte{255}))
 
 				for j := 0; j < samplesCount; j++ {
 					inner := rand.Intn(ref + 1)
@@ -272,7 +290,7 @@ func TestReadToEndWithCheckpoint(t *testing.T) {
 							V:   float64(i),
 						},
 					}, nil)
-					testutil.Ok(t, w.Log(sample))
+					require.NoError(t, w.Log(sample))
 				}
 			}
 
@@ -287,7 +305,7 @@ func TestReadToEndWithCheckpoint(t *testing.T) {
 						Labels: labels.Labels{labels.Label{Name: "__name__", Value: fmt.Sprintf("metric_%d", i)}},
 					},
 				}, nil)
-				testutil.Ok(t, w.Log(series))
+				require.NoError(t, w.Log(series))
 
 				for j := 0; j < samplesCount; j++ {
 					sample := enc.Samples([]record.RefSample{
@@ -297,12 +315,12 @@ func TestReadToEndWithCheckpoint(t *testing.T) {
 							V:   float64(i),
 						},
 					}, nil)
-					testutil.Ok(t, w.Log(sample))
+					require.NoError(t, w.Log(sample))
 				}
 			}
 
-			_, _, err = w.Segments()
-			testutil.Ok(t, err)
+			_, _, err = Segments(w.Dir())
+			require.NoError(t, err)
 			wt := newWriteToMock()
 			watcher := NewWatcher(wMetrics, nil, nil, "", wt, dir)
 			go watcher.Start()
@@ -312,7 +330,7 @@ func TestReadToEndWithCheckpoint(t *testing.T) {
 				return wt.checkNumLabels() >= expected
 			})
 			watcher.Stop()
-			testutil.Equals(t, expected, wt.checkNumLabels())
+			require.Equal(t, expected, wt.checkNumLabels())
 		})
 	}
 }
@@ -325,18 +343,23 @@ func TestReadCheckpoint(t *testing.T) {
 	for _, compress := range []bool{false, true} {
 		t.Run(fmt.Sprintf("compress=%t", compress), func(t *testing.T) {
 			dir, err := ioutil.TempDir("", "readCheckpoint")
-			testutil.Ok(t, err)
-			defer os.RemoveAll(dir)
+			require.NoError(t, err)
+			defer func() {
+				require.NoError(t, os.RemoveAll(dir))
+			}()
 
 			wdir := path.Join(dir, "wal")
 			err = os.Mkdir(wdir, 0777)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 
 			os.Create(SegmentName(wdir, 30))
 
 			enc := record.Encoder{}
 			w, err := NewSize(nil, nil, wdir, 128*pageSize, compress)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
+			defer func() {
+				require.NoError(t, w.Close())
+			}()
 
 			// Write to the initial segment then checkpoint.
 			for i := 0; i < seriesCount; i++ {
@@ -347,7 +370,7 @@ func TestReadCheckpoint(t *testing.T) {
 						Labels: labels.Labels{labels.Label{Name: "__name__", Value: fmt.Sprintf("metric_%d", i)}},
 					},
 				}, nil)
-				testutil.Ok(t, w.Log(series))
+				require.NoError(t, w.Log(series))
 
 				for j := 0; j < samplesCount; j++ {
 					inner := rand.Intn(ref + 1)
@@ -358,15 +381,15 @@ func TestReadCheckpoint(t *testing.T) {
 							V:   float64(i),
 						},
 					}, nil)
-					testutil.Ok(t, w.Log(sample))
+					require.NoError(t, w.Log(sample))
 				}
 			}
 			Checkpoint(log.NewNopLogger(), w, 30, 31, func(x uint64) bool { return true }, 0)
 			w.Truncate(32)
 
 			// Start read after checkpoint, no more data written.
-			_, _, err = w.Segments()
-			testutil.Ok(t, err)
+			_, _, err = Segments(w.Dir())
+			require.NoError(t, err)
 
 			wt := newWriteToMock()
 			watcher := NewWatcher(wMetrics, nil, nil, "", wt, dir)
@@ -377,7 +400,7 @@ func TestReadCheckpoint(t *testing.T) {
 				return wt.checkNumLabels() >= expectedSeries
 			})
 			watcher.Stop()
-			testutil.Equals(t, expectedSeries, wt.checkNumLabels())
+			require.Equal(t, expectedSeries, wt.checkNumLabels())
 		})
 	}
 }
@@ -392,16 +415,18 @@ func TestReadCheckpointMultipleSegments(t *testing.T) {
 	for _, compress := range []bool{false, true} {
 		t.Run(fmt.Sprintf("compress=%t", compress), func(t *testing.T) {
 			dir, err := ioutil.TempDir("", "readCheckpoint")
-			testutil.Ok(t, err)
-			defer os.RemoveAll(dir)
+			require.NoError(t, err)
+			defer func() {
+				require.NoError(t, os.RemoveAll(dir))
+			}()
 
 			wdir := path.Join(dir, "wal")
 			err = os.Mkdir(wdir, 0777)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 
 			enc := record.Encoder{}
 			w, err := NewSize(nil, nil, wdir, pageSize, compress)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 
 			// Write a bunch of data.
 			for i := 0; i < segments; i++ {
@@ -413,7 +438,7 @@ func TestReadCheckpointMultipleSegments(t *testing.T) {
 							Labels: labels.Labels{labels.Label{Name: "__name__", Value: fmt.Sprintf("metric_%d", j)}},
 						},
 					}, nil)
-					testutil.Ok(t, w.Log(series))
+					require.NoError(t, w.Log(series))
 
 					for k := 0; k < samplesCount; k++ {
 						inner := rand.Intn(ref + 1)
@@ -424,19 +449,19 @@ func TestReadCheckpointMultipleSegments(t *testing.T) {
 								V:   float64(i),
 							},
 						}, nil)
-						testutil.Ok(t, w.Log(sample))
+						require.NoError(t, w.Log(sample))
 					}
 				}
 			}
-			testutil.Ok(t, w.Close())
+			require.NoError(t, w.Close())
 
 			// At this point we should have at least 6 segments, lets create a checkpoint dir of the first 5.
 			checkpointDir := dir + "/wal/checkpoint.000004"
 			err = os.Mkdir(checkpointDir, 0777)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 			for i := 0; i <= 4; i++ {
 				err := os.Rename(SegmentName(dir+"/wal", i), SegmentName(checkpointDir, i))
-				testutil.Ok(t, err)
+				require.NoError(t, err)
 			}
 
 			wt := newWriteToMock()
@@ -447,10 +472,10 @@ func TestReadCheckpointMultipleSegments(t *testing.T) {
 			watcher.setMetrics()
 
 			lastCheckpoint, _, err := LastCheckpoint(watcher.walDir)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 
 			err = watcher.readCheckpoint(lastCheckpoint)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 		})
 	}
 }
@@ -472,16 +497,21 @@ func TestCheckpointSeriesReset(t *testing.T) {
 	for _, tc := range testCases {
 		t.Run(fmt.Sprintf("compress=%t", tc.compress), func(t *testing.T) {
 			dir, err := ioutil.TempDir("", "seriesReset")
-			testutil.Ok(t, err)
-			defer os.RemoveAll(dir)
+			require.NoError(t, err)
+			defer func() {
+				require.NoError(t, os.RemoveAll(dir))
+			}()
 
 			wdir := path.Join(dir, "wal")
 			err = os.Mkdir(wdir, 0777)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 
 			enc := record.Encoder{}
 			w, err := NewSize(nil, nil, wdir, segmentSize, tc.compress)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
+			defer func() {
+				require.NoError(t, w.Close())
+			}()
 
 			// Write to the initial segment, then checkpoint later.
 			for i := 0; i < seriesCount; i++ {
@@ -492,7 +522,7 @@ func TestCheckpointSeriesReset(t *testing.T) {
 						Labels: labels.Labels{labels.Label{Name: "__name__", Value: fmt.Sprintf("metric_%d", i)}},
 					},
 				}, nil)
-				testutil.Ok(t, w.Log(series))
+				require.NoError(t, w.Log(series))
 
 				for j := 0; j < samplesCount; j++ {
 					inner := rand.Intn(ref + 1)
@@ -503,12 +533,12 @@ func TestCheckpointSeriesReset(t *testing.T) {
 							V:   float64(i),
 						},
 					}, nil)
-					testutil.Ok(t, w.Log(sample))
+					require.NoError(t, w.Log(sample))
 				}
 			}
 
-			_, _, err = w.Segments()
-			testutil.Ok(t, err)
+			_, _, err = Segments(w.Dir())
+			require.NoError(t, err)
 
 			wt := newWriteToMock()
 			watcher := NewWatcher(wMetrics, nil, nil, "", wt, dir)
@@ -519,24 +549,24 @@ func TestCheckpointSeriesReset(t *testing.T) {
 			retry(t, defaultRetryInterval, defaultRetries, func() bool {
 				return wt.checkNumLabels() >= expected
 			})
-			testutil.Equals(t, seriesCount, wt.checkNumLabels())
+			require.Equal(t, seriesCount, wt.checkNumLabels())
 
 			_, err = Checkpoint(log.NewNopLogger(), w, 2, 4, func(x uint64) bool { return true }, 0)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 
 			err = w.Truncate(5)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 
 			_, cpi, err := LastCheckpoint(path.Join(dir, "wal"))
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 			err = watcher.garbageCollectSeries(cpi + 1)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 
 			watcher.Stop()
 			// If you modify the checkpoint and truncate segment #'s run the test to see how
 			// many series records you end up with and change the last Equals check accordingly
 			// or modify the Equals to Assert(len(wt.seriesLabels) < seriesCount*10)
-			testutil.Equals(t, tc.segments, wt.checkNumLabels())
+			require.Equal(t, tc.segments, wt.checkNumLabels())
 		})
 	}
 }
