@@ -35,13 +35,14 @@ import (
 	"github.com/go-kit/kit/log"
 	"github.com/gogo/protobuf/proto"
 	"github.com/golang/snappy"
+	"github.com/pkg/errors"
 	"github.com/prometheus/client_golang/prometheus"
 	config_util "github.com/prometheus/common/config"
 	"github.com/prometheus/common/model"
 	"github.com/prometheus/common/promlog"
 	"github.com/prometheus/common/route"
+	"github.com/stretchr/testify/require"
 
-	"github.com/pkg/errors"
 	"github.com/prometheus/prometheus/config"
 	"github.com/prometheus/prometheus/pkg/gate"
 	"github.com/prometheus/prometheus/pkg/labels"
@@ -55,8 +56,8 @@ import (
 	"github.com/prometheus/prometheus/storage"
 	"github.com/prometheus/prometheus/storage/remote"
 	"github.com/prometheus/prometheus/tsdb"
+	"github.com/prometheus/prometheus/tsdb/chunks"
 	"github.com/prometheus/prometheus/util/teststorage"
-	"github.com/prometheus/prometheus/util/testutil"
 )
 
 // testMetaStore satisfies the scrape.MetricMetadataStore interface.
@@ -309,10 +310,10 @@ func TestEndpoints(t *testing.T) {
 			test_metric4{foo="boo", dup="1"} 1+0x100
 			test_metric4{foo="boo"} 1+0x100
 	`)
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 	defer suite.Close()
 
-	testutil.Ok(t, suite.Run())
+	require.NoError(t, suite.Run())
 
 	now := time.Now()
 
@@ -349,13 +350,13 @@ func TestEndpoints(t *testing.T) {
 		defer server.Close()
 
 		u, err := url.Parse(server.URL)
-		testutil.Ok(t, err)
+		require.NoError(t, err)
 
 		al := promlog.AllowedLevel{}
-		testutil.Ok(t, al.Set("debug"))
+		require.NoError(t, al.Set("debug"))
 
 		af := promlog.AllowedFormat{}
-		testutil.Ok(t, af.Set("logfmt"))
+		require.NoError(t, af.Set("logfmt"))
 
 		promlogConfig := promlog.Config{
 			Level:  &al,
@@ -363,10 +364,12 @@ func TestEndpoints(t *testing.T) {
 		}
 
 		dbDir, err := ioutil.TempDir("", "tsdb-api-ready")
-		testutil.Ok(t, err)
+		require.NoError(t, err)
 		defer os.RemoveAll(dbDir)
 
-		remote := remote.NewStorage(promlog.New(&promlogConfig), prometheus.DefaultRegisterer, nil, dbDir, 1*time.Second)
+		remote := remote.NewStorage(promlog.New(&promlogConfig), prometheus.DefaultRegisterer, func() (int64, error) {
+			return 0, nil
+		}, dbDir, 1*time.Second, nil)
 
 		err = remote.ApplyConfig(&config.Config{
 			RemoteReadConfigs: []*config.RemoteReadConfig{
@@ -377,7 +380,7 @@ func TestEndpoints(t *testing.T) {
 				},
 			},
 		})
-		testutil.Ok(t, err)
+		require.NoError(t, err)
 
 		var algr rulesRetrieverMock
 		algr.testing = t
@@ -415,9 +418,9 @@ func TestLabelNames(t *testing.T) {
 			test_metric2{foo="boo"} 1+0x100
 			test_metric2{foo="boo", xyz="qwerty"} 1+0x100
 	`)
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 	defer suite.Close()
-	testutil.Ok(t, suite.Run())
+	require.NoError(t, suite.Run())
 
 	api := &API{
 		Queryable: suite.Storage(),
@@ -433,7 +436,7 @@ func TestLabelNames(t *testing.T) {
 	for _, method := range []string{http.MethodGet, http.MethodPost} {
 		ctx := context.Background()
 		req, err := request(method)
-		testutil.Ok(t, err)
+		require.NoError(t, err)
 		res := api.labelNames(req.WithContext(ctx))
 		assertAPIError(t, res.err, "")
 		assertAPIResponse(t, res.data, []string{"__name__", "baz", "foo", "foo1", "foo2", "xyz"})
@@ -724,6 +727,13 @@ func testEndpoints(t *testing.T, api *API, tr *testTargetRetriever, testLabelAPI
 				labels.FromStrings("__name__", "test_metric2", "foo", "boo"),
 			},
 		},
+		{
+			endpoint: api.series,
+			query: url.Values{
+				"match[]": []string{`{foo=""}`},
+			},
+			errType: errorBadData,
+		},
 		{
 			endpoint: api.series,
 			query: url.Values{
@@ -1612,7 +1622,84 @@ func testEndpoints(t *testing.T, api *API, tr *testTargetRetriever, testLabelAPI
 					"boo",
 				},
 			},
-
+			// Label values with bad matchers.
+			{
+				endpoint: api.labelValues,
+				params: map[string]string{
+					"name": "foo",
+				},
+				query: url.Values{
+					"match[]": []string{`{foo=""`, `test_metric2`},
+				},
+				errType: errorBadData,
+			},
+			// Label values with empty matchers.
+			{
+				endpoint: api.labelValues,
+				params: map[string]string{
+					"name": "foo",
+				},
+				query: url.Values{
+					"match[]": []string{`{foo=""}`},
+				},
+				errType: errorBadData,
+			},
+			// Label values with matcher.
+			{
+				endpoint: api.labelValues,
+				params: map[string]string{
+					"name": "foo",
+				},
+				query: url.Values{
+					"match[]": []string{`test_metric2`},
+				},
+				response: []string{
+					"boo",
+				},
+			},
+			// Label values with matcher.
+			{
+				endpoint: api.labelValues,
+				params: map[string]string{
+					"name": "foo",
+				},
+				query: url.Values{
+					"match[]": []string{`test_metric1`},
+				},
+				response: []string{
+					"bar",
+					"boo",
+				},
+			},
+			// Label values with matcher using label filter.
+			{
+				endpoint: api.labelValues,
+				params: map[string]string{
+					"name": "foo",
+				},
+				query: url.Values{
+					"match[]": []string{`test_metric1{foo="bar"}`},
+				},
+				response: []string{
+					"bar",
+				},
+			},
+			// Label values with matcher and time range.
+			{
+				endpoint: api.labelValues,
+				params: map[string]string{
+					"name": "foo",
+				},
+				query: url.Values{
+					"match[]": []string{`test_metric1`},
+					"start":   []string{"1"},
+					"end":     []string{"100000000"},
+				},
+				response: []string{
+					"bar",
+					"boo",
+				},
+			},
 			// Label names.
 			{
 				endpoint: api.labelNames,
@@ -1698,6 +1785,60 @@ func testEndpoints(t *testing.T, api *API, tr *testTargetRetriever, testLabelAPI
 				},
 				response: []string{"__name__", "dup", "foo"},
 			},
+			// Label names with bad matchers.
+			{
+				endpoint: api.labelNames,
+				query: url.Values{
+					"match[]": []string{`{foo=""`, `test_metric2`},
+				},
+				errType: errorBadData,
+			},
+			// Label values with empty matchers.
+			{
+				endpoint: api.labelNames,
+				params: map[string]string{
+					"name": "foo",
+				},
+				query: url.Values{
+					"match[]": []string{`{foo=""}`},
+				},
+				errType: errorBadData,
+			},
+			// Label names with matcher.
+			{
+				endpoint: api.labelNames,
+				query: url.Values{
+					"match[]": []string{`test_metric2`},
+				},
+				response: []string{"__name__", "foo"},
+			},
+			// Label names with matcher.
+			{
+				endpoint: api.labelNames,
+				query: url.Values{
+					"match[]": []string{`test_metric3`},
+				},
+				response: []string{"__name__", "dup", "foo"},
+			},
+			// Label names with matcher using label filter.
+			// There is no matching series.
+			{
+				endpoint: api.labelNames,
+				query: url.Values{
+					"match[]": []string{`test_metric1{foo="test"}`},
+				},
+				response: []string{},
+			},
+			// Label names with matcher and time range.
+			{
+				endpoint: api.labelNames,
+				query: url.Values{
+					"match[]": []string{`test_metric2`},
+					"start":   []string{"1"},
+					"end":     []string{"100000000"},
+				},
+				response: []string{"__name__", "foo"},
+			},
 		}...)
 	}
 
@@ -1784,23 +1925,7 @@ func assertAPIError(t *testing.T, got *apiError, exp errorType) {
 func assertAPIResponse(t *testing.T, got interface{}, exp interface{}) {
 	t.Helper()
 
-	if !reflect.DeepEqual(exp, got) {
-		respJSON, err := json.Marshal(got)
-		if err != nil {
-			t.Fatalf("failed to marshal response as JSON: %v", err.Error())
-		}
-
-		expectedRespJSON, err := json.Marshal(exp)
-		if err != nil {
-			t.Fatalf("failed to marshal expected response as JSON: %v", err.Error())
-		}
-
-		t.Fatalf(
-			"Response does not match, expected:\n%+v\ngot:\n%+v",
-			string(expectedRespJSON),
-			string(respJSON),
-		)
-	}
+	require.Equal(t, exp, got)
 }
 
 func assertAPIResponseLength(t *testing.T, got interface{}, expLen int) {
@@ -1821,12 +1946,12 @@ func TestSampledReadEndpoint(t *testing.T) {
 		load 1m
 			test_metric1{foo="bar",baz="qux"} 1
 	`)
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	defer suite.Close()
 
 	err = suite.Run()
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	api := &API{
 		Queryable:   suite.Storage(),
@@ -1849,21 +1974,21 @@ func TestSampledReadEndpoint(t *testing.T) {
 
 	// Encode the request.
 	matcher1, err := labels.NewMatcher(labels.MatchEqual, "__name__", "test_metric1")
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	matcher2, err := labels.NewMatcher(labels.MatchEqual, "d", "e")
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	query, err := remote.ToQuery(0, 1, []*labels.Matcher{matcher1, matcher2}, &storage.SelectHints{Step: 0, Func: "avg"})
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	req := &prompb.ReadRequest{Queries: []*prompb.Query{query}}
 	data, err := proto.Marshal(req)
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	compressed := snappy.Encode(nil, data)
 	request, err := http.NewRequest("POST", "", bytes.NewBuffer(compressed))
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	recorder := httptest.NewRecorder()
 	api.remoteRead(recorder, request)
@@ -1872,25 +1997,25 @@ func TestSampledReadEndpoint(t *testing.T) {
 		t.Fatal(recorder.Code)
 	}
 
-	testutil.Equals(t, "application/x-protobuf", recorder.Result().Header.Get("Content-Type"))
-	testutil.Equals(t, "snappy", recorder.Result().Header.Get("Content-Encoding"))
+	require.Equal(t, "application/x-protobuf", recorder.Result().Header.Get("Content-Type"))
+	require.Equal(t, "snappy", recorder.Result().Header.Get("Content-Encoding"))
 
 	// Decode the response.
 	compressed, err = ioutil.ReadAll(recorder.Result().Body)
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	uncompressed, err := snappy.Decode(nil, compressed)
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	var resp prompb.ReadResponse
 	err = proto.Unmarshal(uncompressed, &resp)
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	if len(resp.Results) != 1 {
 		t.Fatalf("Expected 1 result, got %d", len(resp.Results))
 	}
 
-	testutil.Equals(t, &prompb.QueryResult{
+	require.Equal(t, &prompb.QueryResult{
 		Timeseries: []*prompb.TimeSeries{
 			{
 				Labels: []prompb.Label{
@@ -1916,11 +2041,11 @@ func TestStreamReadEndpoint(t *testing.T) {
             test_metric1{foo="bar2",baz="qux"} 0+100x120
             test_metric1{foo="bar3",baz="qux"} 0+100x240
 	`)
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	defer suite.Close()
 
-	testutil.Ok(t, suite.Run())
+	require.NoError(t, suite.Run())
 
 	api := &API{
 		Queryable:   suite.Storage(),
@@ -1945,13 +2070,13 @@ func TestStreamReadEndpoint(t *testing.T) {
 
 	// Encode the request.
 	matcher1, err := labels.NewMatcher(labels.MatchEqual, "__name__", "test_metric1")
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	matcher2, err := labels.NewMatcher(labels.MatchEqual, "d", "e")
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	matcher3, err := labels.NewMatcher(labels.MatchEqual, "foo", "bar1")
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	query1, err := remote.ToQuery(0, 14400001, []*labels.Matcher{matcher1, matcher2}, &storage.SelectHints{
 		Step:  1,
@@ -1959,7 +2084,7 @@ func TestStreamReadEndpoint(t *testing.T) {
 		Start: 0,
 		End:   14400001,
 	})
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	query2, err := remote.ToQuery(0, 14400001, []*labels.Matcher{matcher1, matcher3}, &storage.SelectHints{
 		Step:  1,
@@ -1967,18 +2092,18 @@ func TestStreamReadEndpoint(t *testing.T) {
 		Start: 0,
 		End:   14400001,
 	})
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	req := &prompb.ReadRequest{
 		Queries:               []*prompb.Query{query1, query2},
 		AcceptedResponseTypes: []prompb.ReadRequest_ResponseType{prompb.ReadRequest_STREAMED_XOR_CHUNKS},
 	}
 	data, err := proto.Marshal(req)
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	compressed := snappy.Encode(nil, data)
 	request, err := http.NewRequest("POST", "", bytes.NewBuffer(compressed))
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	recorder := httptest.NewRecorder()
 	api.remoteRead(recorder, request)
@@ -1987,8 +2112,8 @@ func TestStreamReadEndpoint(t *testing.T) {
 		t.Fatal(recorder.Code)
 	}
 
-	testutil.Equals(t, "application/x-streamed-protobuf; proto=prometheus.ChunkedReadResponse", recorder.Result().Header.Get("Content-Type"))
-	testutil.Equals(t, "", recorder.Result().Header.Get("Content-Encoding"))
+	require.Equal(t, "application/x-streamed-protobuf; proto=prometheus.ChunkedReadResponse", recorder.Result().Header.Get("Content-Type"))
+	require.Equal(t, "", recorder.Result().Header.Get("Content-Encoding"))
 
 	var results []*prompb.ChunkedReadResponse
 	stream := remote.NewChunkedReader(recorder.Result().Body, remote.DefaultChunkedReadLimit, nil)
@@ -1998,7 +2123,7 @@ func TestStreamReadEndpoint(t *testing.T) {
 		if err == io.EOF {
 			break
 		}
-		testutil.Ok(t, err)
+		require.NoError(t, err)
 		results = append(results, res)
 	}
 
@@ -2006,7 +2131,7 @@ func TestStreamReadEndpoint(t *testing.T) {
 		t.Fatalf("Expected 5 result, got %d", len(results))
 	}
 
-	testutil.Equals(t, []*prompb.ChunkedReadResponse{
+	require.Equal(t, []*prompb.ChunkedReadResponse{
 		{
 			ChunkedSeries: []*prompb.ChunkedSeries{
 				{
@@ -2142,7 +2267,7 @@ func (f *fakeDB) Stats(statsByLabelName string) (_ *tsdb.Stats, retErr error) {
 			retErr = err
 		}
 	}()
-	h, _ := tsdb.NewHead(nil, nil, nil, 1000, "", nil, tsdb.DefaultStripeSize, nil)
+	h, _ := tsdb.NewHead(nil, nil, nil, 1000, "", nil, chunks.DefaultWriteBufferSize, tsdb.DefaultStripeSize, nil)
 	return h.Stats(statsByLabelName), nil
 }
 
@@ -2310,7 +2435,7 @@ func TestAdminEndpoints(t *testing.T) {
 		tc := tc
 		t.Run("", func(t *testing.T) {
 			dir, _ := ioutil.TempDir("", "fakeDB")
-			defer func() { testutil.Ok(t, os.RemoveAll(dir)) }()
+			defer func() { require.NoError(t, os.RemoveAll(dir)) }()
 
 			api := &API{
 				db:          tc.db,
@@ -2321,7 +2446,7 @@ func TestAdminEndpoints(t *testing.T) {
 
 			endpoint := tc.endpoint(api)
 			req, err := http.NewRequest(tc.method, fmt.Sprintf("?%s", tc.values.Encode()), nil)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 
 			res := setUnavailStatusOnTSDBNotReady(endpoint(req))
 			assertAPIError(t, res.err, tc.errType)
@@ -2362,9 +2487,7 @@ func TestRespondSuccess(t *testing.T) {
 		Status: statusSuccess,
 		Data:   "test",
 	}
-	if !reflect.DeepEqual(&res, exp) {
-		t.Fatalf("Expected response \n%v\n but got \n%v\n", res, exp)
-	}
+	require.Equal(t, exp, &res)
 }
 
 func TestRespondError(t *testing.T) {
@@ -2402,9 +2525,7 @@ func TestRespondError(t *testing.T) {
 		ErrorType: errorTimeout,
 		Error:     "message",
 	}
-	if !reflect.DeepEqual(&res, exp) {
-		t.Fatalf("Expected response \n%v\n but got \n%v\n", res, exp)
-	}
+	require.Equal(t, exp, &res)
 }
 
 func TestParseTimeParam(t *testing.T) {
@@ -2414,7 +2535,7 @@ func TestParseTimeParam(t *testing.T) {
 	}
 
 	ts, err := parseTime("1582468023986")
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	var tests = []struct {
 		paramName    string
@@ -2456,15 +2577,15 @@ func TestParseTimeParam(t *testing.T) {
 
 	for _, test := range tests {
 		req, err := http.NewRequest("GET", "localhost:42/foo?"+test.paramName+"="+test.paramValue, nil)
-		testutil.Ok(t, err)
+		require.NoError(t, err)
 
 		result := test.result
 		asTime, err := parseTimeParam(req, test.paramName, test.defaultValue)
 
 		if err != nil {
-			testutil.ErrorEqual(t, result.asError(), err)
+			require.EqualError(t, err, result.asError().Error())
 		} else {
-			testutil.Assert(t, asTime.Equal(result.asTime), "time as return value: %s not parsed correctly. Expected %s. Actual %s", test.paramValue, result.asTime, asTime)
+			require.True(t, asTime.Equal(result.asTime), "time as return value: %s not parsed correctly. Expected %s. Actual %s", test.paramValue, result.asTime, asTime)
 		}
 	}
 }
@@ -2776,8 +2897,8 @@ func TestReturnAPIError(t *testing.T) {
 
 	for _, c := range cases {
 		actual := returnAPIError(c.err)
-		testutil.NotOk(t, actual)
-		testutil.Equals(t, c.expected, actual.typ)
+		require.Error(t, actual)
+		require.Equal(t, c.expected, actual.typ)
 	}
 }
 
