@@ -7,12 +7,17 @@ export interface Target {
   labels: Labels;
   scrapePool: string;
   scrapeUrl: string;
+  globalUrl: string;
   lastError: string;
   lastScrape: string;
   lastScrapeDuration: number;
   health: string;
 }
 
+export interface DroppedTarget {
+  discoveredLabels: Labels;
+}
+
 export interface ScrapePool {
   upCount: number;
   targets: Target[];
